import { useEffect, useRef } from "react";
import styles from "./GuessPane.module.css";

const tileSize = 20;
const tiles = 32;
const rawSize = tileSize * tiles;

// A standard normal can be approximated by 12 summed uniform random values, shifted and rescaled
// https://en.wikipedia.org/wiki/Irwin%E2%80%93Hall_distribution#Approximating_a_Normal_distribution
// This can then be reshifted to a better normal distribution for apple locations
// https://en.wikipedia.org/wiki/Normal_distribution#General_normal_distribution
// Specifically, desired mean is tiles / 2, with a std deviation of tiles / 4
// (tiles / 2) + (tiles / 4) * (sum(U_(0, 1), 12) - 6)
// = (tiles / 2) + ((tiles / 4) * sum(U_(0, 1), 12) - (tiles * 3 / 2))
// = (tiles / 4) * sum(U_(0, 1), 12) - tiles
// Then, take round and clamp that to [0, tiles - 1] (which does technically change the dist a bit)
const tileFactor = tiles / 4;
const gaussianRandomTile = () => {
  const rands = Array.from({ length: 12 }, () => Math.random()).reduce(
    (x, y) => x + y
  );
  const rescaled = Math.round(tileFactor * rands - tiles);
  return Math.max(0, Math.min(tiles - 1, rescaled));
};

const OPPOSITE_KEY = {
  ArrowUp: "ArrowDown",
  ArrowDown: "ArrowUp",
  ArrowLeft: "ArrowRight",
  ArrowRight: "ArrowLeft",
};

const drawTile = (ctx, x, y, color) => {
  ctx.fillStyle = color;
  ctx.fillRect(x * tileSize, y * tileSize, tileSize, tileSize);
};

const snakeGame = () => {
  const snakeHead = { x: null, y: null, vx: null, vy: null };
  let snakeTail = [];
  const apple = { x: null, y: null };

  const randomApple = () => {
    apple.x = gaussianRandomTile();
    apple.y = gaussianRandomTile();
  };

  const reset = () => {
    snakeHead.x = Math.floor(tiles / 2);
    snakeHead.y = snakeHead.x;
    snakeHead.vx = 0;
    snakeHead.vy = -1;
    snakeTail = [{ ...snakeHead }];
    randomApple();
  };

  reset();

  let lastKey;

  const onKey = ({ code }) => {
    if (snakeTail.length <= 1 || OPPOSITE_KEY[lastKey] !== code) {
      lastKey = code;
    }
  };

  let interval;

  const start = (ctx, finishScore, onFinish) => {
    document.addEventListener("keydown", onKey);
    interval = setInterval(() => {
      switch (lastKey) {
        case "ArrowUp":
          snakeHead.vx = 0;
          snakeHead.vy = -1;
          break;
        case "ArrowDown":
          snakeHead.vx = 0;
          snakeHead.vy = 1;
          break;
        case "ArrowLeft":
          snakeHead.vx = -1;
          snakeHead.vy = 0;
          break;
        case "ArrowRight":
          snakeHead.vx = 1;
          snakeHead.vy = 0;
          break;
        default:
          break;
      }

      snakeHead.x += snakeHead.vx;
      snakeHead.y += snakeHead.vy;

      if (
        snakeHead.x < 0 ||
        snakeHead.x >= tiles ||
        snakeHead.y < 0 ||
        snakeHead.y >= tiles ||
        snakeTail.find(({ x, y }) => x === snakeHead.x && y === snakeHead.y)
      ) {
        reset();
      }

      snakeTail.push({ ...snakeHead });

      if (apple.x === snakeHead.x && apple.y === snakeHead.y) {
        randomApple();
      } else {
        snakeTail.shift();
      }

      const finished = snakeTail.length >= finishScore;

      ctx.fillStyle = "black";
      ctx.fillRect(0, 0, rawSize, rawSize);

      snakeTail.forEach(({ x, y }) => {
        drawTile(ctx, x, y, "green");
      });

      if (!finished) {
        drawTile(ctx, apple.x, apple.y, "red");
      }
      ctx.font = "24px serif";
      ctx.fillStyle = "#fff";
      ctx.textAlign = "left";
      ctx.textBaseline = "top";
      ctx.fillText(`${snakeTail.length}/${finishScore}`, tileSize, tileSize);

      if (finished) {
        clearInterval(interval);
        interval = null;

        ctx.font = "48px serif";
        ctx.textAlign = "center";
        ctx.textBaseline = "middle";
        ctx.fillText("Done!", rawSize / 2, rawSize / 2);

        setTimeout(onFinish, 500);
      }
    }, 60);
  };

  const stop = () => {
    document.removeEventListener("keydown", onKey);
    if (interval) {
      clearInterval(interval);
    }
  };

  return { start, stop, onKey };
};

const GunGame = ({ onFinish }) => {
  const canvasRef = useRef();
  useEffect(() => {
    const { start, stop } = snakeGame();
    start(canvasRef.current?.getContext("2d"), 10, onFinish);
    return () => {
      stop();
    };
  }, [onFinish]);

  return (
    <div className={styles.gungame}>
      <div className={styles.gungame_panel}>
        <div className={styles.gungame_label}>
          <div>Not good enough! Finish this snake game to try again!</div>
        </div>
        <canvas
          className={styles.gungame_canvas}
          ref={canvasRef}
          width={rawSize}
          height={rawSize}
        />
      </div>
    </div>
  );
};

export default GunGame;
